/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/
package net.moioli.chart;

import java.util.HashSet;

/**
 * An AbstractChart is a Collection of drawable objects (instances of classes
 * implementing the DrawableI interface). You can use all the Collection methods
 * on an AbstractChart, but probably only DrawableI objects will be drawn.
 * <br />
 * You just have to implement the drawMe() method, where you'll most probably
 * draw the Axes or a Legend after having created an appropriate Scale object.
 * For an example implementation, see the DefaultChart class.
 * 
 * @see net.moioli.chart.DrawableI
 * @see net.moioli.chart.GraphicsI
 * @author Silvio Moioli
 * @version 2005/10/5
 */
public abstract class AbstractChart extends HashSet implements ChartI {
    /**
     * To draw mathematical graphs, DefaultChart will calculate the value of
     * every function multiple times. This parameter assigns the the number of
     * pixels on the x axis after which the y value will be recalculated. 1 is
     * the minimum possible value (maximum precision), 3 is recommended in most
     * cases.
     */
    private int pixelAccuracy = 3;

    /** Minimum X value represented in this AbstractChart. */
    private double minX = -5;

    /** Maximum X value represented in this AbstractChart. */
    private double maxX = 5;

    /**
     * Minimum Y value represented in this AbstractChart. This can be NaN for
     * auto-scaling.
     */
    private double minY = Double.NaN;

    /**
     * Maximum Y value represented in this AbstractChart. This can be NaN for
     * auto-scaling.
     */
    private double maxY = Double.NaN;

    /**
     * Default constructor.
     */
    public AbstractChart() {
        super();
    }

    /**
     * @see net.moioli.chart.ChartI#drawMe(net.moioli.chart.GraphicsI,
     *      net.moioli.chart.IntRectangle)
     */
    public abstract void drawMe(GraphicsI g, IntRectangle clip);

    // Getters and Setters
    /**
     * @see net.moioli.chart.ChartI#getMaxX()
     */
    public double getMaxX() {
        return this.maxX;
    }

    /**
     * @see net.moioli.chart.ChartI#setMaxX(double)
     */
    public void setMaxX(double maxX) {
        this.maxX = maxX;
    }

    /**
     * @see net.moioli.chart.ChartI#getMinX()
     */
    public double getMinX() {
        return this.minX;
    }

    /**
     * @see net.moioli.chart.ChartI#setMinX(double)
     */
    public void setMinX(double minX) {
        this.minX = minX;
    }

    /**
     * @see net.moioli.chart.ChartI#getMaxY()
     */
    public double getMaxY() {
        return this.maxY;
    }

    /**
     * @see net.moioli.chart.ChartI#setMaxY(double)
     */
    public void setMaxY(double maxY) {
        this.maxY = maxY;
    }

    /**
     * @see net.moioli.chart.ChartI#getMinY()
     */
    public double getMinY() {
        return this.minY;
    }

    /**
     * @see net.moioli.chart.ChartI#setMinY(double)
     */
    public void setMinY(double minY) {
        this.minY = minY;
    }

    /**
     * @see net.moioli.chart.ChartI#getPixelAccuracy()
     */
    public int getPixelAccuracy() {
        return this.pixelAccuracy;
    }

    /**
     * @see net.moioli.chart.ChartI#setPixelAccuracy(int)
     */
    public void setPixelAccuracy(int pixelAccuracy) {
        this.pixelAccuracy = pixelAccuracy;
    }

    // Overrides from Collection
    /**
     * @see net.moioli.chart.ChartI#add(java.lang.Object)
     */
    public boolean add(Object arg0) {
        boolean result;
        // We don't want null elements
        if (arg0 != null) {
            result = super.add(arg0);
        } else {
            result = false;
        }
        return result;
    }
}
