/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/

package net.moioli.drawables;

import net.moioli.chart.DrawableI;
import net.moioli.chart.GraphicsI;
import net.moioli.chart.Scale;

/**
 * Represents the x and y axes of the DefaultChart, their measure units, and their
 * legend.
 * 
 * @author Silvio Moioli
 * @version 2005/10/5
 */
public class Axes implements DrawableI {
    /**
     * The arrows' angle.
     */
    private static final double ARROWSANGLE = Math.PI / 6;

    /**
     * This object will divide both x and y axes in a number of segments, each
     * one representing a measure unit described in the Legend, to make the
     * DefaultChart easier to read. This number represents the maximum number of
     * segments on an axis.
     */
    private static final int MAXUNITS = 20;
    
    /**
     * Default constructor.
     */
    public Axes() {
        super();
    }

    /**
     * This object will divide both x and y axes in a number of segments, each
     * one representing a measure unit described in the Legend, to make the
     * DefaultChart easier to read. This method calculates the number of pixels
     * corresponding to this-DefaultChart's-x-axis measure unit.
     * 
     * @return the width of a measure unit in this DefaultChart.
     * @param s
     *            the DefaultChart's Scale object
     */
    public int xMeasureUnitWidth(Scale s) {
        /*
         * If the measure unit would be too tiny (more than MAXUNITS segments on
         * a axis), change it to have exactly MAXUNITS units.
         */
        int result;
        if (((s.getMaxX() - s.getMinX()) < MAXUNITS)) {
            result = (int) Math.round(s.getXZoom());
        }
        else{
            result = (int) Math.round(s.getScreenMaxX() / (double) MAXUNITS);
        }
        return result;
    }

    /**
     * This object will divide both x and y axes in a number of segments, each
     * one representing a measure unit described in the Legend, to make the
     * DefaultChart easier to read. This method calculates the number of pixels
     * corresponding to this-DefaultChart's-y-axis measure unit.
     * 
     * @return the width of a measure unit in this DefaultChart.
     * @param s
     *            the DefaultChart's Scale object
     */
    public int yMeasureUnitWidth(Scale s) {
        /*
         * If the measure unit would be too tiny (more than MAXUNITS segments on
         * a axis), change it to have exactly MAXUNITS units.
         */
        int result;
        if (((s.getMaxY() - s.getMinY()) < MAXUNITS)) {
            result = (int) Math.round(s.getYZoom());
        }
        else{result = (int) Math.round(s.getScreenMaxY() / (double) MAXUNITS);}
        return result;
    }

    /**
     * This object will divide both x and y axes in a number of segments, each
     * one representing a measure unit described in the Legend, to make the
     * DefaultChart easier to read. This calculates the width of an x axis segment on
     * the DefaultChart (not the number of pixels).
     * 
     * @return width of a segment
     * @param s
     *            the DefaultChart's Scale object
     */
    public double xMeasureUnitValue(Scale s) {
        double result;
        if (((s.getMaxX() - s.getMinX()) < MAXUNITS)) {
            result = 1;
        }
        else{
        result = (s.getScreenMaxX() / MAXUNITS) / s.getXZoom();
        }
        return result;
    }

    /**
     * This object will divide both x and y axes in a number of segments, each
     * one representing a measure unit described in the Legend, to make the
     * DefaultChart easier to read. This calculates the width of an y axis segment on
     * the DefaultChart (not the number of pixels).
     * 
     * @return width of a segment
     * @param s
     *            the DefaultChart's Scale object
     */
    public double yMeasureUnitValue(Scale s) {
        double result;
        if (((s.getMaxY() - s.getMinY()) < MAXUNITS)) {
            result = 1;
        }
        result = (s.getScreenMaxY() / MAXUNITS) / s.getYZoom();
        return result;
    }

    /**
     * @see net.moioli.chart.DrawableI#drawMe(GraphicsI, Scale, Object)
     */
    public void drawMe(GraphicsI g, Scale s, Object c) {
        g.setColor(c);

        // Calculates the interval widths and values
        int xWidth = xMeasureUnitWidth(s);
        int yWidth = yMeasureUnitWidth(s);

        double xValue = xMeasureUnitValue(s);
        double yValue = yMeasureUnitValue(s);

        // Calculate the points to draw the axes
        //
        // C.
        // ^
        // |
        // |
        // |
        // A.--------------------------------->.B
        // |O
        // |
        // |
        // |
        // |
        // D.
        //
        DoublePoint A = new DoublePoint(s.getX(s.getMinX()), s.getY(0));

        DoublePoint B = new DoublePoint(s.getX(s.getMaxX()), s.getY(0));

        DoublePoint C = new DoublePoint(s.getX(0), s.getY(s.getMaxY()));

        DoublePoint D = new DoublePoint(s.getX(0), s.getY(s.getMinY()));

        // Draw the axes
        g.drawLine((int) Math.round(A.getX()), (int) Math.round(A.getY()),
                (int) Math.round(B.getX()), (int) Math.round(B.getY()));
        g.drawLine((int) Math.round(C.getX()), (int) Math.round(C.getY()),
                (int) Math.round(D.getX()), (int) Math.round(D.getY()));

        // Calculates the points for the arrows and draws them
        drawArrow(B, Math.PI, g, s);
        drawArrow(C, -Math.PI / 2, g, s);

        // Draw the units' segments
        for (int i = (int) Math.round(C.getX()); i < s.getScreenMaxX(); i += xWidth) {
            g.drawLine(i, (int) Math.round(A.getY() - s.getDetail()), i,
                    (int) Math.round(A.getY() + s.getDetail()));
        }
        for (int i = (int) Math.round(C.getX()); i > 0; i -= xWidth) {
            g.drawLine(i, (int) Math.round(A.getY() - s.getDetail()), i,
                    (int) Math.round(A.getY() + s.getDetail()));
        }
        for (int i = (int) Math.round(A.getY()); i < s.getScreenMaxY(); i += yWidth) {
            g.drawLine((int) Math.round(C.getX() - s.getDetail()), i,
                    (int) Math.round(C.getX() + s.getDetail()), i);
        }
        for (int i = (int) Math.round(A.getY()); i > 0; i -= yWidth) {
            g.drawLine((int) Math.round(C.getX() - s.getDetail()), i,
                    (int) Math.round(C.getX() + s.getDetail()), i);
        }

        // Writes letters on the axes
        g.drawString("x", (int) Math.round(B.getX() - s.getDetail()),
                (int) Math.round(B.getY() + s.getDetail()));
        g.drawString("y", (int) Math.round(C.getX() - s.getDetail()),
                (int) Math.round(C.getY()));

        // Draws the legend
        LegendElement e[] = { new LegendElement("X:", xWidth, xValue),
                new LegendElement("Y:", yWidth, yValue) };
        Legend leg = new Legend(e);
        leg.drawMe(g, s, c);
    }

    /**
     * Draws an arrow head.
     * 
     * @param point
     *            the arrow's head point
     * @param angle
     *            the arrow's head angle
     * @param g
     *            the GraphicsI surface to be used for drawing
     * @param s
     *            the Scale object in this DefaultChart
     */
    private void drawArrow(DoublePoint point, double angle, GraphicsI g, Scale s) {
        int xPoints[] = {
                (int) Math.round(point.getX()),
                (int) Math.round(point.getX() + 2 * s.getDetail()
                        * Math.cos(angle + ARROWSANGLE)),
                (int) Math.round(point.getX() + 2 * s.getDetail()
                        * Math.cos(angle - ARROWSANGLE)) };
        int yPoints[] = {
                (int) Math.round(point.getY()),
                (int) Math.round(point.getY() - 2 * s.getDetail()
                        * Math.sin(angle + ARROWSANGLE)),
                (int) Math.round(point.getY() - 2 * s.getDetail()
                        * Math.sin(angle - ARROWSANGLE)) };
        g.fillPolygon(xPoints, yPoints, 3);
    }

    /**
     * Since the axis will be drawn regardless of the scale, always return NaN.
     * 
     * @param minX
     *            the minimum x value represented in this DefaultChart
     * @param maxX
     *            the maximum x value represented in this DefaultChart
     * @return NaN
     */
    public double maxY(double minX, double maxX) {
        return Double.NaN;
    }

    /**
     * Since the axis will be drawn regardless of the scale, always return NaN.
     * 
     * @param minX
     *            the minimum x value represented in this DefaultChart
     * @param maxX
     *            the maximum x value represented in this DefaultChart
     * @return NaN
     */
    public double minY(double minX, double maxX) {
        return Double.NaN;
    }
}