/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/

package net.moioli.drawables;

import net.moioli.chart.DrawableI;
import net.moioli.chart.GraphicsI;
import net.moioli.chart.Scale;
import java.lang.Double;

/**
 * Represents an element in a DefaultChart's legend.
 * 
 * @author Silvio Moioli
 * @version 2005/10/5
 */
public class LegendElement implements DrawableI {
    /** Pixel length of this LegendElement */
    private int length = 10;

    /** Real value of the segment of this LegendElement */
    private double value = 1;

    /** Other description (if any) */
    private String description = "";

    /** Relative position of this LegendElement (x) */
    private int x = 0;

    /** Relative position of this LegendElement (y) */
    private int y = 0;

    /**
     * Default constructor (empty LegendElement).
     */
    public LegendElement() {
        super();
        this.description = "";
        this.length = 10;
        this.value = 1;
        this.x = 0;
        this.y = 0;
    }

    /**
     * Standard constructor.
     * 
     * @param description
     *            a description string to be drawn near this element
     * @param length
     *            the length (in pixels) of this element
     * @param value
     *            the numeric (real) value of the segment represented by this
     *            element
     */
    public LegendElement(String description, int length, double value) {
        super();
        this.description = description;
        this.length = length;
        this.value = value;
    }

    /**
     * Moves this element changing the x and y relative positioning coordinate.
     * 
     * @param x
     *            the new abscissa (relative to the Legend)
     * @param y
     *            the new ordinate (relative to the Legend
     */
    public void setXY(int x, int y) {
        this.x = x;
        this.y = y;
    }

    /**
     * Implements DrawableI drawing this object.
     * 
     * @param g
     *            the surface where this object will be drawn
     * @param s
     *            the DefaultChart's Scale object
     * @param c
     *            the color to be used for drawing
     */
    public void drawMe(GraphicsI g, Scale s, Object c) {
        // Use this as a measure of the details
        int l = g.getStringWidth(" ");

        int descriptionLength = g.getStringWidth(this.description);
        int descriptionHeight = g.getStringHeight();

        // Calculates the string to be drawn and its offsets
        String stringValue = Double.toString(this.value);
        int charsToBeDrawn = 1;
        int stringValueWidth = g.getStringWidth(stringValue.substring(0,
                charsToBeDrawn));
        while ((stringValueWidth < this.length)
                && (charsToBeDrawn < stringValue.length())) {
            charsToBeDrawn += 1;
            stringValueWidth = g.getStringWidth(stringValue.substring(0,
                    charsToBeDrawn));
        }
        if (stringValue.length() > charsToBeDrawn) {
            stringValue = stringValue.substring(0, charsToBeDrawn);
        }
        int offset = (this.length - (g.getStringWidth(stringValue))) / 2;

        // Calculates the top-left corner of the LegendElement
        int pX = this.x - (descriptionLength + 2 * l + this.length);
        int pY = this.y + descriptionHeight;

        // Do the drawing
        g.drawString(this.description, pX, pY);
        g.drawLine(pX + descriptionLength, pY, pX + descriptionLength, pY + l);
        g.drawLine(pX + descriptionLength + this.length, pY, pX
                + descriptionLength + this.length, pY + l);
        g.drawLine(pX + descriptionLength, pY + l / 2, pX + descriptionLength
                + this.length, pY + l / 2);
        g.drawString(stringValue, pX + descriptionLength + offset, pY);
    }

    /**
     * Return this LegendElement's height.
     * 
     * @param g
     *            the surface where this object will be drawn
     * @return the height
     */
    public int getHeight(GraphicsI g) {
        return g.getStringHeight() * 2;
    }

    /**
     * Since the Legend will be drawn regardless of the scale, always return
     * NaN.
     * 
     * @param minX
     *            the minimum x value represented in this DefaultChart
     * @param maxX
     *            the maximum x value represented in this DefaultChart
     * @return NaN
     */
    public double maxY(double minX, double maxX) {
        return Double.NaN;
    }

    /**
     * Since the Legend will be drawn regardless of the scale, always return
     * NaN.
     * 
     * @param minX
     *            the minimum x value represented in this DefaultChart
     * @param maxX
     *            the maximum x value represented in this DefaultChart
     * @return NaN
     */
    public double minY(double minX, double maxX) {
        return Double.NaN;
    }
}